==> zarf-v0.28.3.tar.gz: zarf-0.28.3/src/test/e2e/08_create_differential_test.go <==
// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: 2021-Present The Zarf Authors

// Package test provides e2e tests for Zarf.
package test

import (
	"fmt"
	"path/filepath"
	"testing"

	"github.com/defenseunicorns/zarf/src/config"
	"github.com/defenseunicorns/zarf/src/pkg/utils"
	"github.com/defenseunicorns/zarf/src/types"
	"github.com/mholt/archiver/v3"
	"github.com/stretchr/testify/require"
)

// TestCreateDifferential creates several differential packages and ensures the already built images and repos and not included in the new package
func TestCreateDifferential(t *testing.T) {
	t.Log("E2E: Test Differential Package Behavior")
	tmpdir := t.TempDir()

	packagePath := "src/test/packages/08-differential-package"
	packageName := "zarf-package-differential-package-amd64-v0.25.0.tar.zst"
	differentialPackageName := "zarf-package-differential-package-amd64-v0.25.0-differential-v0.26.0.tar.zst"
	differentialFlag := fmt.Sprintf("--differential=%s", packageName)

	// Build the package a first time
	stdOut, stdErr, err := e2e.Zarf("package", "create", packagePath, "--set=PACKAGE_VERSION=v0.25.0", "--confirm")
	require.NoError(t, err, stdOut, stdErr)
	defer e2e.CleanFiles(packageName)

	// Build the differential package without changing the version
	_, stdErr, err = e2e.Zarf("package", "create", packagePath, "--set=PACKAGE_VERSION=v0.25.0", differentialFlag, "--confirm")
	require.Error(t, err, "zarf package create should have errored when a differential package was being created without updating the package version number")
	require.Contains(t, stdErr, "unable to create a differential package with the same version")

	// Build the differential package
	stdOut, stdErr, err = e2e.Zarf("package", "create", packagePath, "--set=PACKAGE_VERSION=v0.26.0", differentialFlag, "--confirm")
	require.NoError(t, err, stdOut, stdErr)
	defer e2e.CleanFiles(differentialPackageName)

	// Extract the yaml of the differential package
	err = archiver.Extract(differentialPackageName, config.ZarfYAML, tmpdir)
	require.NoError(t, err, "unable to extract zarf.yaml from the differential git package")

	// Load the extracted zarf.yaml specification
	var differentialZarfConfig types.ZarfPackage
	err = utils.ReadYaml(filepath.Join(tmpdir, config.ZarfYAML), &differentialZarfConfig)
	require.NoError(t, err, "unable to read zarf.yaml from the differential git package")

	// Get a list of all images and repos that are inside of the differential package
	actualGitRepos := []string{}
	actualImages := []string{}
	for _, component := range differentialZarfConfig.Components {
		actualGitRepos = append(actualGitRepos, component.Repos...)
		actualImages = append(actualImages, component.Images...)
	}

	/* Validate we have ONLY the git repos we expect to have */
	expectedGitRepos := []string{
		"https://github.com/stefanprodan/podinfo.git",
		"https://github.com/kelseyhightower/nocode.git",
		"https://github.com/defenseunicorns/zarf.git@refs/tags/v0.26.0",
	}
	require.Len(t, actualGitRepos, 4, "zarf.yaml from the differential package does not contain the correct number of repos")
	for _, expectedRepo := range expectedGitRepos {
		require.Contains(t, actualGitRepos, expectedRepo, fmt.Sprintf("unable to find expected repo %s", expectedRepo))
	}

	/* Validate we have ONLY the images we expect to have */
	expectedImages := []string{
		"ghcr.io/stefanprodan/podinfo:latest",
		"ghcr.io/defenseunicorns/zarf/agent:v0.26.0",
	}
	require.Len(t, actualImages, 2, "zarf.yaml from the differential package does not contain the correct number of images")
	for _, expectedImage := range expectedImages {
		require.Contains(t, actualImages, expectedImage, fmt.Sprintf("unable to find expected image %s", expectedImage))
	}
}

==> zarf-v0.32.1.tar.gz: zarf-0.32.1/src/test/e2e/08_create_differential_test.go <==
// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: 2021-Present The Zarf Authors

// Package test provides e2e tests for Zarf.
package test

import (
	"fmt"
	"path/filepath"
	"testing"

	"github.com/defenseunicorns/zarf/src/pkg/layout"
	"github.com/defenseunicorns/zarf/src/pkg/utils"
	"github.com/defenseunicorns/zarf/src/types"
	"github.com/mholt/archiver/v3"
	"github.com/stretchr/testify/require"
)

// TestCreateDifferential creates several differential packages and ensures the already built images and repos and not included in the new package
func TestCreateDifferential(t *testing.T) {
	t.Log("E2E: Test Differential Package Behavior")
	tmpdir := t.TempDir()
	cachePath := filepath.Join(tmpdir, ".cache-location")

	packagePath := "src/test/packages/08-differential-package"
	packageName := "zarf-package-differential-package-amd64-v0.25.0.tar.zst"
	differentialPackageName := "zarf-package-differential-package-amd64-v0.25.0-differential-v0.26.0.tar.zst"
	differentialFlag := fmt.Sprintf("--differential=%s", packageName)

	// Build the package a first time
	stdOut, stdErr, err := e2e.Zarf("package", "create", packagePath, "--zarf-cache", cachePath, "--set=PACKAGE_VERSION=v0.25.0", "--confirm")
	require.NoError(t, err, stdOut, stdErr)
	defer e2e.CleanFiles(packageName)

	// Build the differential package without changing the version
	_, stdErr, err = e2e.Zarf("package", "create", packagePath, "--zarf-cache", cachePath, "--set=PACKAGE_VERSION=v0.25.0", differentialFlag, "--confirm")
	require.Error(t, err, "zarf package create should have errored when a differential package was being created without updating the package version number")
	require.Contains(t, stdErr, "unable to create a differential package with the same version")

	// Build the differential package
	stdOut, stdErr, err = e2e.Zarf("package", "create", packagePath, "--zarf-cache", cachePath, "--set=PACKAGE_VERSION=v0.26.0", differentialFlag, "--confirm")
	require.NoError(t, err, stdOut, stdErr)
	defer e2e.CleanFiles(differentialPackageName)

	// Extract the yaml of the differential package
	err = archiver.Extract(differentialPackageName, layout.ZarfYAML, tmpdir)
	require.NoError(t, err, "unable to extract zarf.yaml from the differential git package")

	// Load the extracted zarf.yaml specification
	var differentialZarfConfig types.ZarfPackage
	err = utils.ReadYaml(filepath.Join(tmpdir, layout.ZarfYAML), &differentialZarfConfig)
	require.NoError(t, err, "unable to read zarf.yaml from the differential git package")

	// Get a list of all images and repos that are inside of the differential package
	actualGitRepos := []string{}
	actualImages := []string{}
	for _, component := range differentialZarfConfig.Components {
		actualGitRepos = append(actualGitRepos, component.Repos...)
		actualImages = append(actualImages, component.Images...)
	}

	/* Validate we have ONLY the git repos we expect to have */
	expectedGitRepos := []string{
		"https://github.com/stefanprodan/podinfo.git",
		"https://github.com/kelseyhightower/nocode.git",
		"https://github.com/defenseunicorns/zarf.git@refs/tags/v0.26.0",
	}
	require.Len(t, actualGitRepos, 4, "zarf.yaml from the differential package does not contain the correct number of repos")
	for _, expectedRepo := range expectedGitRepos {
		require.Contains(t, actualGitRepos, expectedRepo, fmt.Sprintf("unable to find expected repo %s", expectedRepo))
	}

	/* Validate we have ONLY the images we expect to have */
	expectedImages := []string{
		"ghcr.io/stefanprodan/podinfo:latest",
		"ghcr.io/defenseunicorns/zarf/agent:v0.26.0",
	}
	require.Len(t, actualImages, 2, "zarf.yaml from the differential package does not contain the correct number of images")
	for _, expectedImage := range expectedImages {
		require.Contains(t, actualImages, expectedImage, fmt.Sprintf("unable to find expected image %s", expectedImage))
	}
}
